@@ -5,12 +5,15 @@ import subprocess
 class TestClangDelta(unittest.TestCase):
 
     @classmethod
-    def check_clang_delta(cls, testcase, arguments):
+    def check_clang_delta(cls, testcase, arguments, output_file=None):
         current = os.path.dirname(__file__)
         binary = os.path.join(current, '../clang_delta')
         cmd = '%s %s %s' % (binary, os.path.join(current, testcase), arguments)
         output = subprocess.check_output(cmd, shell=True, encoding='utf8')
-        expected = open(os.path.join(current, os.path.splitext(testcase)[0] + '.output')).read()
+        if output_file:
+            expected = open(os.path.join(current, output_file)).read()
+        else:
+            expected = open(os.path.join(current, os.path.splitext(testcase)[0] + '.output')).read()
         assert output == expected
 
     @classmethod
@@ -21,6 +24,15 @@ class TestClangDelta(unittest.TestCase):
         output = subprocess.check_output(cmd, shell=True, encoding='utf8')
         assert output.strip() == expected
 
+    @classmethod
+    def check_error_message(cls, testcase, arguments, error_message):
+        current = os.path.dirname(__file__)
+        binary = os.path.join(current, '../clang_delta')
+        cmd = '%s %s %s' % (binary, os.path.join(current, testcase), arguments)
+        proc = subprocess.run(cmd, shell=True, encoding='utf8', stdout=subprocess.PIPE)
+        assert proc.returncode == 255
+        assert proc.stdout.strip() == error_message
+
     def test_aggregate_to_scalar_cast(self):
         self.check_clang_delta('aggregate-to-scalar/cast.c', '--transformation=aggregate-to-scalar --counter=1')
 
@@ -94,6 +106,19 @@ class TestClangDelta(unittest.TestCase):
     def test_empty_struct_to_int_test3(self):
         self.check_clang_delta('empty-struct-to-int/test3.c', '--transformation=empty-struct-to-int --counter=1')
 
+    def test_instantiate_template_param_default_param(self):
+        self.check_clang_delta('instantiate-template-param/default_param.cc', '--transformation=instantiate-template-param --counter=1')
+
+    def test_instantiate_template_param_default_test1(self):
+        self.check_clang_delta('instantiate-template-param/test1.cc', '--transformation=instantiate-template-param --counter=1')
+
+    def test_instantiate_template_param_default_test2(self):
+        self.check_clang_delta('instantiate-template-param/test2.cc', '--transformation=instantiate-template-param --counter=1')
+
+    def test_instantiate_template_param_default_test3(self):
+        self.check_query_instances('instantiate-template-param/test3.cc', '--query-instances=instantiate-template-param',
+                'Available transformation instances: 0')
+
     def test_local_to_global_macro(self):
         self.check_clang_delta('local-to-global/macro.c', '--transformation=local-to-global --counter=1')
 
@@ -118,6 +143,103 @@ class TestClangDelta(unittest.TestCase):
     def test_remove_enum_member_value_builtin_macro(self):
         self.check_clang_delta('remove-enum-member-value/builtin_macro.c', '--transformation=remove-enum-member-value --counter=1')
 
+    def test_remove_unused_enum_member_value_range(self):
+        self.check_clang_delta('remove-unused-enum-member/range.c', '--transformation=remove-unused-enum-member --counter=4 --to-counter=9')
+
+    def test_remove_namespace_macro(self):
+        self.check_clang_delta('remove-namespace/macro.cpp', '--transformation=remove-namespace --counter=1')
+        self.check_clang_delta('remove-namespace/macro.cpp', '--transformation=remove-namespace --counter=2',
+                'remove-namespace/macro.output2')
+        self.check_clang_delta('remove-namespace/macro.cpp', '--transformation=remove-namespace --counter=3',
+                'remove-namespace/macro.output3')
+        self.check_clang_delta('remove-namespace/macro.cpp', '--transformation=remove-namespace --counter=4',
+                'remove-namespace/macro.output4')
+        self.check_clang_delta('remove-namespace/macro.cpp', '--transformation=remove-namespace --counter=5',
+                'remove-namespace/macro.output5')
+
+    def test_remove_namespace(self):
+        self.check_clang_delta('remove-namespace/namespace.cpp', '--transformation=remove-namespace --counter=1')
+        self.check_clang_delta('remove-namespace/namespace.cpp', '--transformation=remove-namespace --counter=2',
+                'remove-namespace/namespace.output2')
+        self.check_clang_delta('remove-namespace/namespace.cpp', '--transformation=remove-namespace --counter=3',
+                'remove-namespace/namespace.output3')
+
+    def test_remove_namespace2(self):
+        self.check_clang_delta('remove-namespace/namespace2.cpp', '--transformation=remove-namespace --counter=1')
+        self.check_clang_delta('remove-namespace/namespace2.cpp', '--transformation=remove-namespace --counter=2',
+                'remove-namespace/namespace2.output2')
+
+    def test_remove_namespace3(self):
+        self.check_clang_delta('remove-namespace/namespace3.cpp', '--transformation=remove-namespace --counter=1')
+        self.check_clang_delta('remove-namespace/namespace3.cpp', '--transformation=remove-namespace --counter=2',
+                'remove-namespace/namespace3.output2')
+        self.check_clang_delta('remove-namespace/namespace3.cpp', '--transformation=remove-namespace --counter=3',
+                'remove-namespace/namespace3.output3')
+
+    def test_remove_namespace4(self):
+        self.check_clang_delta('remove-namespace/namespace4.cpp', '--transformation=remove-namespace --counter=1')
+
+    def test_remove_namespace5(self):
+        self.check_clang_delta('remove-namespace/namespace5.cpp', '--transformation=remove-namespace --counter=1')
+        self.check_clang_delta('remove-namespace/namespace5.cpp', '--transformation=remove-namespace --counter=2',
+                'remove-namespace/namespace5.output2')
+        self.check_clang_delta('remove-namespace/namespace5.cpp', '--transformation=remove-namespace --counter=3',
+                'remove-namespace/namespace5.output3')
+        self.check_clang_delta('remove-namespace/namespace5.cpp', '--transformation=remove-namespace --counter=4',
+                'remove-namespace/namespace5.output4')
+
+    def test_remove_namespace6(self):
+        self.check_clang_delta('remove-namespace/namespace6.cpp', '--transformation=remove-namespace --counter=1')
+        self.check_clang_delta('remove-namespace/namespace6.cpp', '--transformation=remove-namespace --counter=2',
+                'remove-namespace/namespace6.output2')
+
+    def test_remove_namespace7(self):
+        self.check_clang_delta('remove-namespace/namespace7.cpp', '--transformation=remove-namespace --counter=1')
+        self.check_clang_delta('remove-namespace/namespace7.cpp', '--transformation=remove-namespace --counter=2',
+                'remove-namespace/namespace7.output2')
+        self.check_clang_delta('remove-namespace/namespace7.cpp', '--transformation=remove-namespace --counter=3',
+                'remove-namespace/namespace7.output3')
+
+    def test_remove_namespace8(self):
+        self.check_clang_delta('remove-namespace/namespace8.cpp', '--transformation=remove-namespace --counter=1')
+        self.check_clang_delta('remove-namespace/namespace8.cpp', '--transformation=remove-namespace --counter=2',
+                'remove-namespace/namespace8.output2')
+
+    def test_remove_namespace9(self):
+        self.check_clang_delta('remove-namespace/namespace9.cpp', '--transformation=remove-namespace --counter=1')
+
+    def test_remove_namespace10(self):
+        self.check_clang_delta('remove-namespace/namespace10.cpp', '--transformation=remove-namespace --counter=1')
+        self.check_clang_delta('remove-namespace/namespace10.cpp', '--transformation=remove-namespace --counter=2',
+                'remove-namespace/namespace10.output2')
+
+    def test_remove_namespace11(self):
+        self.check_clang_delta('remove-namespace/namespace11.cpp', '--transformation=remove-namespace --counter=1')
+        self.check_clang_delta('remove-namespace/namespace11.cpp', '--transformation=remove-namespace --counter=2',
+                'remove-namespace/namespace11.output2')
+
+    def test_remove_namespace12(self):
+        self.check_clang_delta('remove-namespace/namespace12.cpp', '--transformation=remove-namespace --counter=1')
+        self.check_clang_delta('remove-namespace/namespace12.cpp', '--transformation=remove-namespace --counter=2',
+                'remove-namespace/namespace12.output2')
+        self.check_clang_delta('remove-namespace/namespace12.cpp', '--transformation=remove-namespace --counter=3',
+                'remove-namespace/namespace12.output3')
+
+    def test_remove_namespace13(self):
+        self.check_clang_delta('remove-namespace/namespace13.cpp', '--transformation=remove-namespace --counter=1')
+        self.check_clang_delta('remove-namespace/namespace13.cpp', '--transformation=remove-namespace --counter=2',
+                'remove-namespace/namespace13.output2')
+
+    def test_remove_namespace14(self):
+        self.check_clang_delta('remove-namespace/namespace14.cpp', '--transformation=remove-namespace --counter=1')
+
+    def test_remove_namespace15(self):
+        self.check_clang_delta('remove-namespace/namespace15.cpp', '--transformation=remove-namespace --counter=1')
+        self.check_clang_delta('remove-namespace/namespace15.cpp', '--transformation=remove-namespace --counter=2',
+                'remove-namespace/namespace15.output2')
+        self.check_clang_delta('remove-namespace/namespace15.cpp', '--transformation=remove-namespace --counter=3',
+                'remove-namespace/namespace15.output3')
+
     def test_remove_nested_function_remove_nested_func1(self):
         self.check_clang_delta('remove-nested-function/remove_nested_func1.cc', '--transformation=remove-nested-function --counter=1')
 
@@ -154,6 +276,28 @@ class TestClangDelta(unittest.TestCase):
     def test_remove_unused_function_class(self):
         self.check_clang_delta('remove-unused-function/class.cc', '--transformation=remove-unused-function --counter=1')
 
+    def test_remove_unused_function_const(self):
+        self.check_clang_delta('remove-unused-function/const.cc', '--transformation=remove-unused-function --counter=1')
+        self.check_clang_delta('remove-unused-function/const.cc', '--transformation=remove-unused-function --counter=2',
+                'remove-unused-function/const.output2')
+
+    def test_remove_unused_function_default(self):
+        self.check_clang_delta('remove-unused-function/default.cc', '--transformation=remove-unused-function --counter=1')
+        self.check_clang_delta('remove-unused-function/default.cc', '--transformation=remove-unused-function --counter=2',
+                'remove-unused-function/default.output2')
+
+    def test_remove_unused_function_delete(self):
+        self.check_clang_delta('remove-unused-function/delete.cc', '--transformation=remove-unused-function --counter=1')
+
+    def test_remove_unused_function_delete2(self):
+        self.check_clang_delta('remove-unused-function/delete2.cc', '--transformation=remove-unused-function --counter=1')
+        self.check_clang_delta('remove-unused-function/delete2.cc', '--transformation=remove-unused-function --counter=2',
+                'remove-unused-function/delete2.output2')
+        self.check_clang_delta('remove-unused-function/delete2.cc', '--transformation=remove-unused-function --counter=3',
+                'remove-unused-function/delete2.output3')
+        self.check_clang_delta('remove-unused-function/delete2.cc', '--transformation=remove-unused-function --counter=4',
+                'remove-unused-function/delete2.output4')
+
     def test_remove_unused_function_inline_ns(self):
         self.check_query_instances('remove-unused-function/inline_ns.cc', '--query-instances=remove-unused-function',
                 'Available transformation instances: 0')
@@ -299,6 +443,9 @@ class TestClangDelta(unittest.TestCase):
     def test_rename_param_invalid(self):
         self.check_clang_delta('rename-param/invalid.c', '--transformation=rename-param --counter=1')
 
+    def test_rename_param_stuck(self):
+        self.check_error_message('rename-param/stuck.ii', '--transformation=rename-param --counter=1', 'Error: No modification to the transformed program!')
+
     def test_rename_var_rename_var(self):
         self.check_clang_delta('rename-var/rename-var.c', '--transformation=rename-var --counter=1')
 
@@ -353,6 +500,13 @@ class TestClangDelta(unittest.TestCase):
     def test_simplify_if_macro(self):
         self.check_clang_delta('simplify-if/macro.c', '--transformation=simplify-if --counter=1')
 
+    def test_simplify_simple_recursive_template(self):
+        self.check_clang_delta('simplify-recursive-template-instantiation/test.cc', '--transformation=simplify-recursive-template-instantiation --counter=1')
+
+    def test_template_arg_to_int_not_valid_5(self):
+        self.check_query_instances('template-arg-to-int/not_valid5.cc', '--query-instances=template-arg-to-int',
+                'Available transformation instances: 0')
+
     def test_union_to_struct_union1(self):
         self.check_clang_delta('union-to-struct/union1.c', '--transformation=union-to-struct --counter=1')
 
@@ -361,3 +515,12 @@ class TestClangDelta(unittest.TestCase):
 
     def test_union_to_struct_union3(self):
         self.check_clang_delta('union-to-struct/union3.c', '--transformation=union-to-struct --counter=1')
+
+    def test_piggypacking(self):
+        current = os.path.dirname(__file__)
+        binary = os.path.join(current, '../clang_delta')
+        cmd = '%s %s %s' % (binary, os.path.join(current, 'remove-unused-function/macro2.cc'),
+                '--transformation=remove-unused-function --counter=111 --to-counter=222 --warn-on-counter-out-of-bounds --report-instances-count')
+        run = subprocess.run(cmd, shell=True, encoding='utf8', stderr=subprocess.PIPE, stdout=subprocess.PIPE)
+        assert 'Available transformation instances: 1' in run.stderr
+        assert 'Warning: number of transformation instances exceeded' in run.stderr
