@@ -31,8 +31,10 @@ from cvise.utils import statistics
 
 class DeltaTimeFormatter(logging.Formatter):
     def format(self, record):
-        duration = datetime.datetime.utcfromtimestamp(record.relativeCreated / 1000)
-        record.delta = duration.strftime("%H:%M:%S")
+        record.delta = str(datetime.timedelta(seconds=int(record.relativeCreated / 1000)))
+        # pad with one more zero
+        if record.delta[1] == ':':
+            record.delta = '0' + record.delta
         return super().format(record)
 
 script_path = os.path.dirname(os.path.realpath(__file__))
@@ -152,7 +154,7 @@ if __name__ == "__main__":
     parser.add_argument("--die-on-pass-bug", action="store_true", default=False, help="Terminate C-Vise if a pass encounters an otherwise non-fatal problem")
     parser.add_argument("--sllooww", action="store_true", default=False, help="Try harder to reduce, but perhaps take a long time to do so")
     parser.add_argument("--also-interesting", metavar="EXIT_CODE", type=int, help="A process exit code (somewhere in the range 64-113 would be usual) that, when returned by the interestingness test, will cause C-Vise to save a copy of the variant")
-    parser.add_argument("--debug", action="store_true", default=False, help="Print debug information")
+    parser.add_argument("--debug", action="store_true", default=False, help="Print debug information (alias for --log-level=DEBUG)")
     parser.add_argument("--log-level", type=str, choices=["INFO", "DEBUG", "WARNING", "ERROR"], default="INFO", help="Define the verbosity of the logged events")
     parser.add_argument("--log-file", type=str, help="Log events into LOG_FILE instead of stderr. New events are appended to the end of the file")
     parser.add_argument("--no-give-up", action="store_true", default=False, help="Don't give up on a pass that hasn't made progress for {} iterations".format(testing.TestManager.GIVEUP_CONSTANT))
@@ -162,7 +164,7 @@ if __name__ == "__main__":
     parser.add_argument("--remove-pass", help="Remove all instances of the specified passes from the schedule (comma-separated)")
     parser.add_argument("--no-timing", action="store_true", default=False, help="Do not print timestamps about reduction progress")
     parser.add_argument("--timestamp", action="store_true", default=False, help="Print timestamps instead of relative time from a reduction start")
-    parser.add_argument("--timeout", type=int, nargs="?", const=300, help="Interestingness test timeout in seconds")
+    parser.add_argument("--timeout", type=int, nargs="?", default=300, help="Interestingness test timeout in seconds")
     parser.add_argument("--no-cache", action="store_true", default=False, help="Don't cache behavior of passes")
     parser.add_argument("--skip-key-off", action="store_true", default=False, help="Disable skipping the rest of the current pass when \"s\" is pressed")
     parser.add_argument("--max-improvement", metavar="BYTES", type=int, help="Largest improvement in file size from a single transformation that C-Vise should accept (useful only to slow C-Vise down)")
