@@ -8,7 +8,7 @@ class ClexPass(AbstractPass):
     def check_prerequisites(self):
         return self.check_external_program("clex")
 
-    def new(self, test_case):
+    def new(self, test_case, _=None):
         return 0
 
     def advance(self, test_case, state):
