@@ -5,7 +5,7 @@ class IndentPass(AbstractPass):
     def check_prerequisites(self):
         return self.check_external_program("clang-format")
 
-    def new(self, test_case):
+    def new(self, test_case, _=None):
         return 0
 
     def advance(self, test_case, state):
