@@ -9,8 +9,8 @@ import os
 import os.path
 import pebble
 import platform
+import psutil
 import shutil
-import signal
 import subprocess
 import sys
 import tempfile
@@ -106,7 +106,7 @@ class TestEnvironment:
                 return self
 
             # run test script
-            self.exitcode = self.run_test()
+            self.exitcode = self.run_test(False)
             return self
         except OSError as e:
             # this can happen when we clean up temporary files for cancelled processes
@@ -117,11 +117,13 @@ class TestEnvironment:
         finally:
             return self
 
-    def run_test(self):
+    def run_test(self, verbose):
         try:
             os.chdir(self.folder)
-            _, _, returncode = ProcessEventNotifier(self.pid_queue).run_process(self.test_script, shell=True,
-                    stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
+            stdout, stderr, returncode = ProcessEventNotifier(self.pid_queue).run_process(self.test_script, shell=True)
+            if verbose and returncode != 0:
+                logging.debug('stdout:\n' + stdout)
+                logging.debug('stderr:\n' + stderr)
         finally:
             os.chdir(self.pwd)
         return returncode
@@ -261,18 +263,20 @@ class TestManager:
 
         return "".join(diffed_lines)
 
-    def check_sanity(self):
+    def check_sanity(self, verbose=False):
         logging.debug("perform sanity check... ")
 
         folder = tempfile.mkdtemp(prefix=self.TEMP_PREFIX)
         test_env = TestEnvironment(None, 0, self.test_script, folder, None, self.test_cases, None)
         logging.debug("sanity check tmpdir = {}".format(test_env.folder))
 
-        returncode = test_env.run_test()
-        rmfolder(folder)
+        returncode = test_env.run_test(verbose)
         if returncode == 0:
+            rmfolder(folder)
             logging.debug("sanity check successful")
         else:
+            if not self.save_temps:
+                rmfolder(folder)
             raise InsaneTestCaseError(self.test_cases, self.test_script)
 
     def release_folder(self, future):
@@ -299,14 +303,28 @@ class TestManager:
                 active_pids.add(event.pid)
         for pid in active_pids:
             try:
-                os.kill(pid, signal.SIGTERM)
-            except ProcessLookupError:
+                process = psutil.Process(pid)
+                children = process.children(recursive=True)
+                children.append(process)
+                for child in children:
+                    try:
+                        child.terminate()
+                    except psutil.NoSuchProcess:
+                        pass
+            except psutil.NoSuchProcess:
                 pass
 
     def release_future(self, future):
         self.futures.remove(future)
         self.release_folder(future)
 
+    def save_extra_dir(self, test_case_path):
+        extra_dir = self.get_extra_dir("cvise_extra_", self.MAX_EXTRA_DIRS)
+        if extra_dir != None:
+            os.mkdir(extra_dir)
+            shutil.move(test_case_path, extra_dir)
+            logging.info("Created extra directory {} for you to look at later".format(extra_dir))
+
     def process_done_futures(self):
         quit_loop = False
         new_futures = set()
@@ -320,7 +338,8 @@ class TestManager:
                 if future.exception():
                     if type(future.exception()) is TimeoutError:
                         self.timeout_count += 1
-                        logging.debug("Test timed out!")
+                        logging.warning("Test timed out.")
+                        self.save_extra_dir(self.temporary_folders[future])
                         if self.timeout_count >= self.MAX_TIMEOUTS:
                             logging.warning("Maximum number of timeout were reached: %d" % self.MAX_TIMEOUTS)
                             quit_loop = True
@@ -347,11 +366,7 @@ class TestManager:
                         assert test_env.exitcode
                         if (self.also_interesting is not None and
                             test_env.exitcode == self.also_interesting):
-                            extra_dir = self.get_extra_dir("cvise_extra_", self.MAX_EXTRA_DIRS)
-                            if extra_dir != None:
-                                os.mkdir(extra_dir)
-                                shutil.move(test_env.test_case_path, extra_dir)
-                                logging.info("Created extra directory {} for you to look at later".format(extra_dir))
+                            self.save_extra_dir(test_env.test_case_path)
                     elif test_env.result == PassResult.STOP:
                         quit_loop = True
                     elif test_env.result == PassResult.ERROR:
